==> jabber-transport-0.7pre3.tar.gz: jabber-transport-0.7pre3/src/conn.c <==
#include "jserver.h"

/*
this is a container to manage all of the incoming namespaces on connections and set the right handlers
*/

struct namespaces
{
    char *ns;
    xmlstream_onNode handler;
    struct namespaces *next;
};

xmlstream_onNode conn_namespace(int cmd, char *ns, xmlstream_onNode handler)
{
    static struct namespaces *n = NULL;
    struct namespaces *next;

    if(ns == NULL)
        return NULL;

    switch(cmd)
    {
        case CMD_SET: /* saves this ns/handler pair */
            if(conn_namespace(CMD_CHECK,ns,NULL) != NULL)
                return NULL;
            next = malloc(sizeof(struct namespaces));
            next->ns = j_strdup(ns);
            next->next = n;
            next->handler = handler;
            n = next;
            return handler;
            break;
        case CMD_CHECK: /* returns the handler for the given ns or returns NULL */
            next = n;
            while(next != NULL)
            {
                if(j_strcmp(ns,next->ns) == 0)
                    return next->handler;
                next = next->next;
            }
            break;
    }

    return NULL;
}


/* this is the callback on the listener thread whenever there is a new connect */
void conn_connect(thread t, int sock, struct sockaddr_in sa)
{
    thread new;
/*    printf("jserver connection established (fd: %d, ip: %s, port: %d)\n",
                sock, inet_ntoa(sa.sin_addr), ntohs(sa.sin_port));*/
    new = xmlstream_new(sock, (xmlstream_onNode)conn_negotiate);
}

/* whenever there is a new connect and root node received, find the namespace for the stream and setup the right handler */
void conn_negotiate(thread t, xmlnode in, int type)
{
    char *ns, *to;
    tstream ts = (tstream)(t->data);
    int error = 0;
    xmlstream_onNode handler;

    if(type != XMLSTREAM_ROOT)
        return;

    debug("New XML Stream started, locating internal namespace handler");

    ns = xmlnode_get_attrib(in, "xmlns");
    to = xmlnode_get_attrib(in, "to");
    handler = conn_namespace(CMD_CHECK,ns,NULL);

    if(type == XMLSTREAM_ERROR || ns == NULL || to == NULL || handler == NULL || config_names(CMD_CHECK,to) == NULL)
    {
        xmlstream_write_header(t, xmlstream_header(NULL, NULL, NULL));
        if(type == XMLSTREAM_ERROR)
            xmlstream_error(t, "You didn't send valid XML.");
        else if(ns == NULL)
            xmlstream_error(t, "You didn't specify a default xmlns='' namespace.");
        else if(to == NULL)
            xmlstream_error(t, "You didn't send your to='server' attribute.");
        else if(handler == NULL)
            xmlstream_error(t, "This server doesn't know about your namespace.");
        else if(config_names(CMD_CHECK,to) == NULL)
            xmlstream_error(t, "This server is not the one in your to attribute.");
    }else{
        (handler)(t, in, type); /* let the real handler be the welcome party */
        ts->read_evt_data = (void *)handler; /* we have a valid handler for this stream, pass it off to them now */
    }
}

==> jabber-transport-0.8.tar.gz: jabber-transport-0.8/src/conn.c <==
#include "jserver.h"

/*
this is a container to manage all of the incoming namespaces on connections and set the right handlers
*/

struct namespaces
{
    char *ns;
    xmlstream_onNode handler;
    struct namespaces *next;
};

xmlstream_onNode conn_namespace(int cmd, char *ns, xmlstream_onNode handler)
{
    static struct namespaces *n = NULL;
    struct namespaces *next;

    if(ns == NULL)
        return NULL;

    switch(cmd)
    {
        case CMD_SET: /* saves this ns/handler pair */
            if(conn_namespace(CMD_CHECK,ns,NULL) != NULL)
                return NULL;
            next = malloc(sizeof(struct namespaces));
            next->ns = j_strdup(ns);
            next->next = n;
            next->handler = handler;
            n = next;
            return handler;
            break;
        case CMD_CHECK: /* returns the handler for the given ns or returns NULL */
            next = n;
            while(next != NULL)
            {
                if(j_strcmp(ns,next->ns) == 0)
                    return next->handler;
                next = next->next;
            }
            break;
    }

    return NULL;
}


/* this is the callback on the listener thread whenever there is a new connect */
void conn_connect(thread t, int sock, struct sockaddr_in sa)
{
    thread new;
/*    jdebug(ZONE,"jserver connection established (fd: %d, ip: %s, port: %d)\n",
                sock, inet_ntoa(sa.sin_addr), ntohs(sa.sin_port));*/
    new = xmlstream_new(sock, inet_ntoa(sa.sin_addr), (xmlstream_onNode)conn_negotiate);
}

/* whenever there is a new connect and root node received, find the namespace for the stream and setup the right handler */
void conn_negotiate(thread t, xmlnode in, int type)
{
    char *ns, *to;
    tstream ts = (tstream)(t->data);
    xmlstream_onNode handler;

    if(type != XMLSTREAM_ROOT)
        return;

    jdebug(ZONE,"New XML Stream started, locating internal namespace handler");

    ns = xmlnode_get_attrib(in, "xmlns");
    to = xmlnode_get_attrib(in, "to");
    handler = conn_namespace(CMD_CHECK,ns,NULL);

    if(type == XMLSTREAM_ERROR || ns == NULL || to == NULL || handler == NULL || js_names(CMD_CHECK,to) == NULL)
    {
        xmlstream_write_header(t, xmlstream_header(NULL, NULL, NULL));
        if(type == XMLSTREAM_ERROR)
            xmlstream_error(t, "You didn't send valid XML.");
        else if(ns == NULL)
            xmlstream_error(t, "You didn't specify a default xmlns='' namespace.");
        else if(to == NULL)
            xmlstream_error(t, "You didn't send your to='server' attribute.");
        else if(handler == NULL)
            xmlstream_error(t, "This server doesn't know about your namespace.");
        else if(js_names(CMD_CHECK,to) == NULL)
            xmlstream_error(t, "This server is not the one in your to attribute.");
    }else{
        (handler)(t, in, type); /* let the real handler be the welcome party */
        ts->data = (void *)handler; /* we have a valid handler for this stream, pass it off to them now */
    }
}
